#include <cstdlib>
#include <iostream>
#include <fstream>
#include <direct.h>

#define FCEU_LEN 3531265
extern "C" { extern unsigned char fceu[]; }

using namespace std;

#define ptElfOut "boot.elf"

#define foNesromsize 0x000883c0
#define foNesrom     0x000883e0
#define szRomDefault 0x00100010

class IntBE
{
  private:
       char rgbyte[4];
      
  public:
      int ToInt() { return rgbyte[3] | rgbyte[2] << 8 | rgbyte[1] << 16 | rgbyte[0] << 24; }
      IntBE(int intHost) 
      {
             rgbyte[0] = (intHost & 0xFF000000) >> 24;
             rgbyte[1] = (intHost & 0x00FF0000) >> 16;
             rgbyte[2] = (intHost & 0x0000FF00) >> 8;
             rgbyte[3] = (intHost & 0x000000FF);
      }
      IntBE(char *rgbyte)
      {
             for (int ibyte = 0; ibyte < 4; ibyte ++)
                 this->rgbyte[ibyte] = rgbyte[ibyte];
      }
      static IntBE Read(istream &file)
      {
             char rgbyte[4];
             file.read(rgbyte, 4);
             return IntBE(rgbyte);
      }
      void Write(ostream &file)
      {
           file.write(rgbyte, 4);
      }
};

#define Fail() system("pause"); exit(1)

int main(int argc, char *argv[])
{   
    string ptSelf(argv[0]);
    cout << "Mike Tyson's Jungle Beat Injector v0.02\n";
    cout << "usage: " << ptSelf << " <Mike Tyson's Punch-Out! ROM file>\n\n";

    string ptDirSelf(argv[0]);
    for (string::iterator it = ptDirSelf.end(); it > ptDirSelf.begin(); it --)
    {
        if (*it == '/' || *it == '\\')
        {
           ptDirSelf.erase(it, ptDirSelf.end());
           break;
        }
    }
    chdir(ptDirSelf.c_str());
    if (argc < 2)
    {
       cout << "Please specify the path to the Mike Tyson's Punch-Out! ROM file\non the command line.\n";
       Fail();
    }
    string ptRom(argv[1]);
    ifstream fileRom(argv[1], ios::binary | ios::in);
    if (!fileRom.is_open())
    {
       cout << "Could not open ROM file " << ptRom << " for reading!\n";
       Fail();
    }
    
    fileRom.seekg(0, ios::end);
    IntBE nesromsizeNew(fileRom.tellg());
    cout << "New ROM size: " << nesromsizeNew.ToInt() << "\n";
    
    if (nesromsizeNew.ToInt() > szRomDefault)
    {
       cout << "ROM is too big!  The largest ROM I can inject is " << szRomDefault << " bytes.\n";
       Fail();
    }
    ofstream fileOut(ptElfOut, ios::binary | ios::out);
    fileOut.write((char*)fceu, foNesromsize);
    nesromsizeNew.Write(fileOut);
    fileRom.seekg(0);
    fileRom.read((char*)&fceu[foNesrom], nesromsizeNew.ToInt());
    fileOut.write((char*)&fceu[fileOut.tellp()], FCEU_LEN - fileOut.tellp());
    fileOut.close();
    cout << "boot.elf written!\n";
    
    system("PAUSE");
    return 0;
}
